/** \file dm9000.h
\note Ported from the Linux kernel, see dm9000.c there for copyright and license info

The driver supports the original DM9000E, and now the two newer
devices, DM9000A and DM9000B. Define which one you are using
	DM9000_TYPE_E:	original DM9000, with no link change interrupt nor checksum offload
	DM9000_TYPE_A:	analog PHY	
	DM9000_TYPE_B:	digital PHY
	DM9000_TYPE_BI:	industrial temperature range
Two-packet mode is done in software, regardless of the chip type.

More defines:
	DM9000_POLL_CONTROLLER	If you are NOT using interrupts and will poll
your controller to get frames and send queued frames	
	DM9000_PROMISC	Promiscuous mode
	DM9000_ALLMULTI	all-multicast addresses are received
	DM9000_CSUMOFFLOAD	not for the E chip 
	DM9000_PLATF_EXT_PHY	use external PHY, yet unsupported 
	DM9000_LNKCHNGINTERRUPT	generate an interrupt on link state change, yet unsupported 
	DM9000_USEWOLINTERRUPT	use wake-up on LAN, yet unsupported 
	DM9000_SUPPORTPHYACCESS	include PHY access code, yet unsupported 
*/

#ifndef _DM9000_H_
#define _DM9000_H_

#include "idtype.h"

struct DM9000_internals {
	uint16_t rx_pkt_len;
	uint16_t queue_pkt_len;
	uint8_t	tx_pkt_cnt;
	uint8_t	rx_pkt_cnt;
};

void dm9000_init(struct idtype_chip_internals *i, uint8_t *mac_addr, uint16_t *mcast);
int dm9000_do_xmit(struct idtype_chip_internals *i, void *data, uint16_t len);
int dm9000_start_xmit(struct idtype_chip_internals *i);
void dm9000_xmit(struct idtype_chip_internals *i, void *data, uint16_t len);
void dm9000_end_xmit(struct idtype_chip_internals *i, uint16_t total_len);

int dm9000_start_rx(struct idtype_chip_internals *i);
void dm9000_rx(struct idtype_chip_internals *i, void *rdptr, uint16_t RxLen);
void dm9000_end_rx(struct idtype_chip_internals *i);
void dm9000_rx_nomem(struct idtype_chip_internals *i, uint16_t RxLen);
int dm9000_do_rx(struct idtype_chip_internals *i, void *rdptr);

#ifdef DM9000_POLL_CONTROLLER
void dm9000_poll_controller(struct idtype_chip_internals *i);
#else
void dm9000_interrupt(struct idtype_chip_internals *i);
#endif

#endif
