/*
*/

#include "idtype.h"

#ifdef IDTYPE_USE16BUS 
static uint8_t txoddnum;
static uint8_t txodd;
static uint8_t rxoddnum;
static uint8_t rxodd;

void idtype_startread(idtype_chip_address_t chip)
{
	rxoddnum = 0;
}

void idtype_startwrite(idtype_chip_address_t chip)
{
	txoddnum = 0;
}
void idtype_endwrite(idtype_chip_address_t chip)
{
	if(txoddnum)	// a byte was left from last block, total odd number of bytes in frame
		_writedata(chip, txodd);
}

#endif

/* Micros with alignment problems might need 8-bit bus or some checking
and modifications to this approach */
void idtype_readblock(idtype_chip_address_t chip, void *data, uint16_t len)
{
#ifdef IDTYPE_USE16BUS
register int mylen = len>>1;
register uint16_t *where = data;
	
	if(rxoddnum){	// a byte was left from previous block, with odd number of bytes
		*(uint8_t *)data = rxodd;
		where = (uint16_t *)((uint8_t *)data + 1);
		len -= 1;
		mylen = len>>1;
		rxoddnum = 0;
	}
	while(mylen--)
		*(where++) = _readdata(chip);
	if(len&1){
		unsigned int rem = _readdata(chip);
		*(uint8_t *)where = (uint8_t) rem; /// \warning big endian requires modding this
		rxodd = ((unsigned int)rem)>>8; /// \warning big endian requires modding this
		rxoddnum = 1;
	}
#endif
#ifdef IDTYPE_USE8BUS
	while(len--)
		*(data++) = _readdata(chip);
#endif
}
		
void idtype_writeblock(idtype_chip_address_t chip, void *data, uint16_t len)
{
#ifdef IDTYPE_USE16BUS
register int mylen = len>>1;
register uint16_t *from = data;
	
	if(txoddnum){	// a byte was left from previous block, with odd number of bytes
		unsigned int rem;
		len -= 1;
		mylen = len>>1;
		rem = txodd | ((*(uint8_t *)from) << 8); /// \warning big endian requires modding this
		_writedata(chip, rem);
		from = (uint16_t *)((uint8_t *)data + 1);
		txoddnum = 0;
	}
	while(mylen--)
		_writedata(chip, *(from++));
	if(len&1){
		txoddnum = 1;
		txodd = *(uint8_t *)from;
	}
#endif
#ifdef IDTYPE_USE8BUS
	while(len--)
		_writedata(chip, *(data++));
#endif
}

